import { RX_NUMBER } from '../constants/regex';
import { File } from '../constants/safe-types';

// --- Convenience inspection utilities ---

export var toType = value => typeof value;
export var toRawType = value => Object.prototype.toString.call(value).slice(8, -1);
export var toRawTypeLC = value => toRawType(value).toLowerCase();
export var isUndefined = value => value === undefined;
export var isNull = value => value === null;
export var isEmptyString = value => value === '';
export var isUndefinedOrNull = value => isUndefined(value) || isNull(value);
export var isUndefinedOrNullOrEmpty = value => isUndefinedOrNull(value) || isEmptyString(value);
export var isFunction = value => toType(value) === 'function';
export var isBoolean = value => toType(value) === 'boolean';
export var isString = value => toType(value) === 'string';
export var isNumber = value => toType(value) === 'number';
export var isNumeric = value => RX_NUMBER.test(String(value));
export var isPrimitive = value => isBoolean(value) || isString(value) || isNumber(value);
export var isArray = value => Array.isArray(value);

// Quick object check
// This is primarily used to tell Objects from primitive values
// when we know the value is a JSON-compliant type
// Note object could be a complex type like array, Date, etc.
export var isObject = obj => obj !== null && typeof obj === 'object';

// Strict object type check
// Only returns true for plain JavaScript objects
export var isPlainObject = obj => Object.prototype.toString.call(obj) === '[object Object]';
export var isDate = value => value instanceof Date;
export var isEvent = value => value instanceof Event;
export var isFile = value => value instanceof File;
export var isRegExp = value => toRawType(value) === 'RegExp';
export var isPromise = value => !isUndefinedOrNull(value) && isFunction(value.then) && isFunction(value.catch);